summaryrefslogtreecommitdiffstats
path: root/src/jogl
diff options
context:
space:
mode:
Diffstat (limited to 'src/jogl')
-rw-r--r--src/jogl/classes/com/jogamp/opengl/JoglVersion.java17
-rw-r--r--src/jogl/classes/com/jogamp/opengl/math/FloatUtil.java26
-rw-r--r--src/jogl/classes/com/jogamp/opengl/math/FovHVHalves.java20
-rw-r--r--src/jogl/classes/com/jogamp/opengl/math/Matrix4.java8
-rw-r--r--src/jogl/classes/com/jogamp/opengl/math/geom/Frustum.java75
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/PMVMatrix.java8
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/glsl/ShaderCode.java330
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/EyeParameter.java4
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/LocationSensorParameter.java51
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/StereoClientRenderer.java7
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDevice.java93
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDeviceRenderer.java22
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/StereoGLEventListener.java6
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/StereoUtil.java41
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/ViewerPose.java (renamed from src/jogl/classes/com/jogamp/opengl/util/stereo/EyePose.java)31
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceConfig.java13
-rw-r--r--src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceFactory.java3
-rw-r--r--src/jogl/classes/jogamp/opengl/ProjectFloat.java3
-rw-r--r--src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDevice.java83
-rw-r--r--src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDeviceRenderer.java46
-rw-r--r--src/jogl/native/openmax/jogamp_opengl_util_av_impl_OMXGLMediaPlayer.c2
-rw-r--r--src/jogl/native/openmax/omx_tool.h1
22 files changed, 726 insertions, 164 deletions
diff --git a/src/jogl/classes/com/jogamp/opengl/JoglVersion.java b/src/jogl/classes/com/jogamp/opengl/JoglVersion.java
index af16348a8..7589b3776 100644
--- a/src/jogl/classes/com/jogamp/opengl/JoglVersion.java
+++ b/src/jogl/classes/com/jogamp/opengl/JoglVersion.java
@@ -129,7 +129,14 @@ public class JoglVersion extends JogampVersion {
public static StringBuilder getGLInfo(final GL gl, final StringBuilder sb) {
return getGLInfo(gl, sb, false);
}
- public static StringBuilder getGLInfo(final GL gl, StringBuilder sb, final boolean withCapabilitiesAndExtensionInfo) {
+ public static StringBuilder getGLInfo(final GL gl, final StringBuilder sb, final boolean withCapabilitiesAndExtensionInfo) {
+ return getGLInfo(gl, sb, true, withCapabilitiesAndExtensionInfo, withCapabilitiesAndExtensionInfo);
+ }
+
+ public static StringBuilder getGLInfo(final GL gl, StringBuilder sb,
+ final boolean withAvailabilityInfo,
+ final boolean withCapabilitiesInfo,
+ final boolean withExtensionInfo) {
final AbstractGraphicsDevice device = gl.getContext().getGLDrawable().getNativeSurface()
.getGraphicsConfiguration().getScreen().getDevice();
if(null==sb) {
@@ -139,12 +146,14 @@ public class JoglVersion extends JogampVersion {
sb.append(VersionUtil.SEPERATOR).append(Platform.getNewline());
sb.append(device.getClass().getSimpleName()).append("[type ")
.append(device.getType()).append(", connection ").append(device.getConnection()).append("]: ").append(Platform.getNewline());
- GLProfile.glAvailabilityToString(device, sb, "\t", 1);
+ if( withAvailabilityInfo ) {
+ GLProfile.glAvailabilityToString(device, sb, "\t", 1);
+ }
sb.append(Platform.getNewline());
- sb = getGLStrings(gl, sb, withCapabilitiesAndExtensionInfo);
+ sb = getGLStrings(gl, sb, withExtensionInfo);
- if( withCapabilitiesAndExtensionInfo ) {
+ if( withCapabilitiesInfo ) {
sb = getAllAvailableCapabilitiesInfo(device, sb);
}
return sb;
diff --git a/src/jogl/classes/com/jogamp/opengl/math/FloatUtil.java b/src/jogl/classes/com/jogamp/opengl/math/FloatUtil.java
index a04055430..7f630f9d5 100644
--- a/src/jogl/classes/com/jogamp/opengl/math/FloatUtil.java
+++ b/src/jogl/classes/com/jogamp/opengl/math/FloatUtil.java
@@ -35,6 +35,7 @@ import jogamp.opengl.Debug;
import com.jogamp.common.os.Platform;
import com.jogamp.opengl.math.geom.AABBox;
+import com.jogamp.opengl.math.geom.Frustum;
/**
* Basic Float math utility functions.
@@ -491,18 +492,18 @@ public final class FloatUtil {
* @param zNear
* @param zFar
* @return given matrix for chaining
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0,
- * or if left == right, or bottom == top, or zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * or {@code left == right}, or {@code bottom == top}.
*/
public static float[] makeFrustum(final float[] m, final int m_offset, final boolean initM,
final float left, final float right,
final float bottom, final float top,
final float zNear, final float zFar) throws GLException {
- if( zNear <= 0.0f || zFar < 0.0f ) {
- throw new GLException("GL_INVALID_VALUE: zNear and zFar must be positive, and zNear>0");
+ if( zNear <= 0.0f || zFar <= zNear ) {
+ throw new GLException("Requirements zNear > 0 and zFar > zNear, but zNear "+zNear+", zFar "+zFar);
}
- if( left == right || top == bottom || zNear == zFar ) {
- throw new GLException("GL_INVALID_VALUE: top,bottom and left,right and zNear,zFar must not be equal");
+ if( left == right || top == bottom) {
+ throw new GLException("GL_INVALID_VALUE: top,bottom and left,right must not be equal");
}
if( initM ) {
// m[m_offset+0+4*0] = 1f;
@@ -565,14 +566,15 @@ public final class FloatUtil {
* @param zNear
* @param zFar
* @return given matrix for chaining
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0, or if zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * @see #makeFrustum(float[], int, boolean, float, float, float, float, float, float)
*/
public static float[] makePerspective(final float[] m, final int m_off, final boolean initM,
final float fovy_rad, final float aspect, final float zNear, final float zFar) throws GLException {
final float top = tan(fovy_rad/2f) * zNear; // use tangent of half-fov !
- final float bottom = -1.0f * top;
- final float left = aspect * bottom;
- final float right = aspect * top;
+ final float bottom = -1.0f * top; // -1f * fovhvTan.top * zNear
+ final float left = aspect * bottom; // aspect * -1f * fovhvTan.top * zNear
+ final float right = aspect * top; // aspect * fovhvTan.top * zNear
return makeFrustum(m, m_off, initM, left, right, bottom, top, zNear, zFar);
}
@@ -591,7 +593,9 @@ public final class FloatUtil {
* @param zNear
* @param zFar
* @return given matrix for chaining
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0, or if zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * @see #makeFrustum(float[], int, boolean, float, float, float, float, float, float)
+ * @see Frustum#updateByFovDesc(float[], int, boolean, Frustum.FovDesc)
*/
public static float[] makePerspective(final float[] m, final int m_offset, final boolean initM,
final FovHVHalves fovhv, final float zNear, final float zFar) throws GLException {
diff --git a/src/jogl/classes/com/jogamp/opengl/math/FovHVHalves.java b/src/jogl/classes/com/jogamp/opengl/math/FovHVHalves.java
index 26ed57009..5d7907f2f 100644
--- a/src/jogl/classes/com/jogamp/opengl/math/FovHVHalves.java
+++ b/src/jogl/classes/com/jogamp/opengl/math/FovHVHalves.java
@@ -35,13 +35,13 @@ package com.jogamp.opengl.math;
* </p>
*/
public final class FovHVHalves {
- /** Half horizontal FOV from center to left. */
+ /** Half horizontal FOV from center to left, either in {@link #inTangents} or radians. */
public final float left;
- /** Half horizontal FOV from center to right. */
+ /** Half horizontal FOV from center to right, either in {@link #inTangents} or radians. */
public final float right;
- /** Half vertical FOV from center to top. */
+ /** Half vertical FOV from center to top, either in {@link #inTangents} or radians. */
public final float top;
- /** Half vertical FOV from center to bottom. */
+ /** Half vertical FOV from center to bottom, either in {@link #inTangents} or radians. */
public final float bottom;
/** If true, values are in tangent, otherwise radians.*/
public final boolean inTangents;
@@ -67,7 +67,7 @@ public final class FovHVHalves {
}
/**
- * Returns a symmetrical centered {@link FovHVHalves} instance in tangents, using:
+ * Returns a symmetrical centered {@link FovHVHalves} instance in {@link #inTangents}, using:
* <pre>
halfHorizFovTan = tan( horizontalFov / 2f );
halfVertFovTan = tan( verticalFov / 2f );
@@ -82,7 +82,7 @@ public final class FovHVHalves {
}
/**
- * Returns a symmetrical centered {@link FovHVHalves} instance in tangents, using:
+ * Returns a symmetrical centered {@link FovHVHalves} instance in {@link #inTangents}, using:
* <pre>
top = bottom = tan( verticalFov / 2f );
left = right = aspect * top;
@@ -99,7 +99,7 @@ public final class FovHVHalves {
}
/**
- * Returns a custom symmetry {@link FovHVHalves} instance in tangents, using:
+ * Returns a custom symmetry {@link FovHVHalves} instance {@link #inTangents}, using:
* <pre>
left = tan( horizontalFov * horizCenterFromLeft )
right = tan( horizontalFov * ( 1f - horizCenterFromLeft ) )
@@ -121,7 +121,7 @@ public final class FovHVHalves {
}
/**
- * Returns a custom symmetry {@link FovHVHalves} instance in tangents,
+ * Returns a custom symmetry {@link FovHVHalves} instance {@link #inTangents},
* via computing the <code>horizontalFov</code> using:
* <pre>
halfVertFovTan = tan( verticalFov / 2f );
@@ -157,10 +157,10 @@ public final class FovHVHalves {
}
}
- /** Returns the full horizontal FOV, i.e. {@link #left} + {@link #right}. */
+ /** Returns the full horizontal FOV, i.e. {@link #left} + {@link #right}, either in {@link #inTangents} or radians. */
public final float horzFov() { return left+right; }
- /** Returns the full vertical FOV, i.e. {@link #top} + {@link #bottom}. */
+ /** Returns the full vertical FOV, i.e. {@link #top} + {@link #bottom}, either in {@link #inTangents} or radians. */
public final float vertFov() { return top+bottom; }
public final String toString() {
diff --git a/src/jogl/classes/com/jogamp/opengl/math/Matrix4.java b/src/jogl/classes/com/jogamp/opengl/math/Matrix4.java
index 421bb909f..a080d4442 100644
--- a/src/jogl/classes/com/jogamp/opengl/math/Matrix4.java
+++ b/src/jogl/classes/com/jogamp/opengl/math/Matrix4.java
@@ -147,8 +147,9 @@ public class Matrix4 {
* @param top
* @param zNear
* @param zFar
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0,
- * or if left == right, or bottom == top, or zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * or {@code left == right}, or {@code bottom == top}.
+ * @see FloatUtil#makeFrustum(float[], int, boolean, float, float, float, float, float, float)
*/
public final void makeFrustum(final float left, final float right, final float bottom, final float top, final float zNear, final float zFar) throws GLException {
multMatrix( FloatUtil.makeFrustum(mat4Tmp1, 0, true, left, right, bottom, top, zNear, zFar) );
@@ -159,7 +160,8 @@ public class Matrix4 {
* @param aspect
* @param zNear
* @param zFar
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0, or if zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * @see FloatUtil#makePerspective(float[], int, boolean, float, float, float, float)
*/
public final void makePerspective(final float fovy_rad, final float aspect, final float zNear, final float zFar) throws GLException {
multMatrix( FloatUtil.makePerspective(mat4Tmp1, 0, true, fovy_rad, aspect, zNear, zFar) );
diff --git a/src/jogl/classes/com/jogamp/opengl/math/geom/Frustum.java b/src/jogl/classes/com/jogamp/opengl/math/geom/Frustum.java
index b73bad613..8b0fa559e 100644
--- a/src/jogl/classes/com/jogamp/opengl/math/geom/Frustum.java
+++ b/src/jogl/classes/com/jogamp/opengl/math/geom/Frustum.java
@@ -30,6 +30,8 @@ package com.jogamp.opengl.math.geom;
import jogamp.common.os.PlatformPropsImpl;
import com.jogamp.common.os.Platform;
+import com.jogamp.opengl.math.FloatUtil;
+import com.jogamp.opengl.math.FovHVHalves;
/**
* Providing frustum {@link #getPlanes() planes} derived by different inputs
@@ -77,6 +79,35 @@ import com.jogamp.common.os.Platform;
* </p>
*/
public class Frustum {
+ /**
+ * {@link Frustum} description by {@link #fovhv} and {@link #zNear}, {@link #zFar}.
+ */
+ public static class FovDesc {
+ /** Field of view in both directions, may not be centered, either {@link FovHVHalves#inTangents} or radians. */
+ public final FovHVHalves fovhv;
+ /** Near Z */
+ public final float zNear;
+ /** Far Z */
+ public final float zFar;
+ /**
+ * @param fovhv field of view in both directions, may not be centered, either {@link FovHVHalves#inTangents} or radians
+ * @param zNear
+ * @param zFar
+ * @throws IllegalArgumentException if {@code zNear <= 0} or {@code zFar <= zNear}.
+ */
+ public FovDesc(final FovHVHalves fovhv, final float zNear, final float zFar) throws IllegalArgumentException {
+ if( zNear <= 0.0f || zFar <= zNear ) {
+ throw new IllegalArgumentException("Requirements zNear > 0 and zFar > zNear, but zNear "+zNear+", zFar "+zFar);
+ }
+ this.fovhv = fovhv;
+ this.zNear = zNear;
+ this.zFar = zFar;
+ }
+ public final String toString() {
+ return "FrustumFovDesc["+fovhv.toStringInDegrees()+", Z["+zNear+" - "+zFar+"]]";
+ }
+ }
+
/** Normalized planes[l, r, b, t, n, f] */
protected final Plane[] planes = new Plane[6];
@@ -176,19 +207,47 @@ public class Frustum {
*/
public final void updateByPlanes(final Plane[] src) {
for (int i = 0; i < 6; ++i) {
- final Plane p0 = planes[i];
- final float[] p0_n = p0.n;
- final Plane p1 = src[i];
- final float[] p1_n = p1.n;
- p0_n[0] = p1_n[0];
- p0_n[1] = p1_n[1];
- p0_n[2] = p1_n[2];
- p0.d = p1.d;
+ final Plane pD = planes[i];
+ final Plane pS = src[i];
+ pD.d = pS.d;
+ System.arraycopy(pS.n, 0, pD.n, 0, 3);
}
}
/**
* Calculate the frustum planes in world coordinates
+ * using the passed {@link FovDesc}.
+ * <p>
+ * Operation Details:
+ * <ul>
+ * <li>The given {@link FovDesc} will be transformed
+ * into the given float[16] as a perspective matrix (column major order) first,
+ * see {@link FloatUtil#makePerspective(float[], int, boolean, FovHVHalves, float, float)}.</li>
+ * <li>Then the float[16] perspective matrix is used to {@link #updateByPMV(float[], int)} this instance.</li>
+ * </ul>
+ * </p>
+ * <p>
+ * Frustum plane's normals will point to the inside of the viewing frustum,
+ * as required by this class.
+ * </p>
+ *
+ * @param m 4x4 matrix in column-major order (also result)
+ * @param m_offset offset in given array <i>m</i>, i.e. start of the 4x4 matrix
+ * @param initM if true, given matrix will be initialized w/ identity matrix,
+ * otherwise only the frustum fields are set.
+ * @param fovDesc {@link Frustum} {@link FovDesc}
+ * @return given matrix for chaining
+ * @see FloatUtil#makePerspective(float[], int, boolean, FovHVHalves, float, float)
+ */
+ public float[] updateByFovDesc(final float[] m, final int m_offset, final boolean initM,
+ final FovDesc fovDesc) {
+ FloatUtil.makePerspective(m, m_offset, initM, fovDesc.fovhv, fovDesc.zNear, fovDesc.zFar);
+ updateByPMV(m, 0);
+ return m;
+ }
+
+ /**
+ * Calculate the frustum planes in world coordinates
* using the passed float[16] as premultiplied P*MV (column major order).
* <p>
* Frustum plane's normals will point to the inside of the viewing frustum,
diff --git a/src/jogl/classes/com/jogamp/opengl/util/PMVMatrix.java b/src/jogl/classes/com/jogamp/opengl/util/PMVMatrix.java
index 57f9301b8..196acf9ab 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/PMVMatrix.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/PMVMatrix.java
@@ -663,8 +663,9 @@ public final class PMVMatrix implements GLMatrixFunc {
/**
* {@inheritDoc}
*
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0,
- * or if left == right, or bottom == top, or zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * or {@code left == right}, or {@code bottom == top}.
+ * @see FloatUtil#makeFrustum(float[], int, boolean, float, float, float, float, float, float)
*/
@Override
public final void glFrustumf(final float left, final float right, final float bottom, final float top, final float zNear, final float zFar) throws GLException {
@@ -682,7 +683,8 @@ public final class PMVMatrix implements GLMatrixFunc {
* @param aspect aspect ratio width / height
* @param zNear
* @param zFar
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0, or if zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * @see FloatUtil#makePerspective(float[], int, boolean, float, float, float, float)
*/
public final void gluPerspective(final float fovy_deg, final float aspect, final float zNear, final float zFar) throws GLException {
glMultMatrixf( FloatUtil.makePerspective(mat4Tmp1, 0, true, fovy_deg * FloatUtil.PI / 180.0f, aspect, zNear, zFar), 0 );
diff --git a/src/jogl/classes/com/jogamp/opengl/util/glsl/ShaderCode.java b/src/jogl/classes/com/jogamp/opengl/util/glsl/ShaderCode.java
index f8a66fe45..e1db3f7a3 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/glsl/ShaderCode.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/glsl/ShaderCode.java
@@ -126,7 +126,7 @@ public class ShaderCode {
* @param source CharSequence array containing the shader sources, organized as <code>source[count][strings-per-shader]</code>.
* May be either an immutable <code>String</code> - or mutable <code>StringBuilder</code> array.
*
- * @throws IllegalArgumentException if <code>count</count> and <code>source.length</code> do not match
+ * @throws IllegalArgumentException if <code>count</code> and <code>source.length</code> do not match
*/
public ShaderCode(final int type, final int count, final CharSequence[][] source) {
if(source.length != count) {
@@ -189,7 +189,7 @@ public class ShaderCode {
* {@link GL4#GL_TESS_CONTROL_SHADER} or {@link GL4#GL_TESS_EVALUATION_SHADER}.
* @param count number of shaders
* @param context class used to help resolving the source location
- * @param sourceFiles array of source locations, organized as <code>sourceFiles[count]</code>
+ * @param sourceFiles array of source locations, organized as <code>sourceFiles[count]</code> -> <code>shaderSources[count][1]</code>
* @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
* which can be edited later on at the costs of a String conversion when passing to
* {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
@@ -197,10 +197,11 @@ public class ShaderCode {
* which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
* at no additional costs.
*
- * @throws IllegalArgumentException if <code>count</count> and <code>sourceFiles.length</code> do not match
+ * @throws IllegalArgumentException if <code>count</code> and <code>sourceFiles.length</code> do not match
* @see #readShaderSource(Class, String)
*/
- public static ShaderCode create(final GL2ES2 gl, final int type, final int count, final Class<?> context, final String[] sourceFiles, final boolean mutableStringBuilder) {
+ public static ShaderCode create(final GL2ES2 gl, final int type, final int count, final Class<?> context,
+ final String[] sourceFiles, final boolean mutableStringBuilder) {
if(null != gl && !ShaderUtil.isShaderCompilerAvailable(gl)) {
return null;
}
@@ -227,6 +228,53 @@ public class ShaderCode {
}
/**
+ * Creates a complete {@link ShaderCode} object while reading all shader sources from {@link Uri} <code>sourceLocations</code>
+ * via {@link #readShaderSource(Uri, boolean)}.
+ *
+ * @param gl current GL object to determine whether a shader compiler is available. If null, no validation is performed.
+ * @param type either {@link GL2ES2#GL_VERTEX_SHADER}, {@link GL2ES2#GL_FRAGMENT_SHADER}, {@link GL3#GL_GEOMETRY_SHADER},
+ * {@link GL4#GL_TESS_CONTROL_SHADER} or {@link GL4#GL_TESS_EVALUATION_SHADER}.
+ * @param count number of shaders
+ * @param sourceLocations array of {@link Uri} source locations, organized as <code>sourceFiles[count]</code> -> <code>shaderSources[count][1]</code>
+ * @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
+ * which can be edited later on at the costs of a String conversion when passing to
+ * {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
+ * If <code>false</code> method returns an immutable <code>String</code> instance,
+ * which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
+ * at no additional costs.
+ *
+ * @throws IllegalArgumentException if <code>count</code> and <code>sourceFiles.length</code> do not match
+ * @see #readShaderSource(Uri, boolean)
+ * @since 2.3.2
+ */
+ public static ShaderCode create(final GL2ES2 gl, final int type, final int count,
+ final Uri[] sourceLocations, final boolean mutableStringBuilder) {
+ if(null != gl && !ShaderUtil.isShaderCompilerAvailable(gl)) {
+ return null;
+ }
+
+ CharSequence[][] shaderSources = null;
+ if(null!=sourceLocations) {
+ // sourceFiles.length and count is validated in ctor
+ shaderSources = new CharSequence[sourceLocations.length][1];
+ for(int i=0; i<sourceLocations.length; i++) {
+ try {
+ shaderSources[i][0] = readShaderSource(sourceLocations[i], mutableStringBuilder);
+ } catch (final IOException ioe) {
+ throw new RuntimeException("readShaderSource("+sourceLocations[i]+") error: ", ioe);
+ }
+ if(null == shaderSources[i][0]) {
+ shaderSources = null;
+ }
+ }
+ }
+ if(null==shaderSources) {
+ return null;
+ }
+ return new ShaderCode(type, count, shaderSources);
+ }
+
+ /**
* Creates a complete {@link ShaderCode} object while reading the shader binary of <code>binaryFile</code>,
* which location is resolved using the <code>context</code> class, see {@link #readShaderBinary(Class, String)}.
*
@@ -259,6 +307,37 @@ public class ShaderCode {
}
/**
+ * Creates a complete {@link ShaderCode} object while reading the shader binary from {@link Uri} <code>binLocations</code>
+ * via {@link #readShaderBinary(Uri)}.
+ * @param type either {@link GL2ES2#GL_VERTEX_SHADER}, {@link GL2ES2#GL_FRAGMENT_SHADER}, {@link GL3#GL_GEOMETRY_SHADER},
+ * {@link GL4#GL_TESS_CONTROL_SHADER} or {@link GL4#GL_TESS_EVALUATION_SHADER}.
+ * @param count number of shaders
+ * @param binFormat a valid native binary format as they can be queried by {@link ShaderUtil#getShaderBinaryFormats(GL)}.
+ * @param binLocations {@link Uri} binary location
+ *
+ * @see #readShaderBinary(Uri)
+ * @see ShaderUtil#getShaderBinaryFormats(GL)
+ * @since 2.3.2
+ */
+ public static ShaderCode create(final int type, final int count, int binFormat, final Uri binLocation) {
+ ByteBuffer shaderBinary = null;
+ if(null!=binLocation && 0<=binFormat) {
+ try {
+ shaderBinary = readShaderBinary(binLocation);
+ } catch (final IOException ioe) {
+ throw new RuntimeException("readShaderBinary("+binLocation+") error: ", ioe);
+ }
+ if(null == shaderBinary) {
+ binFormat = -1;
+ }
+ }
+ if(null==shaderBinary) {
+ return null;
+ }
+ return new ShaderCode(type, count, binFormat, shaderBinary);
+ }
+
+ /**
* Returns a unique suffix for shader resources as follows:
* <ul>
* <li>Source<ul>
@@ -327,7 +406,8 @@ public class ShaderCode {
* whatever is available first.
* <p>
* The source and binary location names are expected w/o suffixes which are
- * resolved and appended using {@link #getFileSuffix(boolean, int)}.
+ * resolved and appended using the given {@code srcSuffixOpt} and {@code binSuffixOpt}
+ * if not {@code null}, otherwise {@link #getFileSuffix(boolean, int)} determines the suffixes.
* </p>
* <p>
* Additionally, the binary resource is expected within a subfolder of <code>binRoot</code>
@@ -358,9 +438,11 @@ public class ShaderCode {
* Your invocation in org/test/glsl/MyShaderTest.java:
*
* ShaderCode vp0 = ShaderCode.create(gl, GL2ES2.GL_VERTEX_SHADER, 1, this.getClass(),
- * "shader", new String[] { "vertex" }, "shader/bin", "vertex");
+ * "shader", new String[] { "vertex" }, null,
+ * "shader/bin", "vertex", null, true);
* ShaderCode fp0 = ShaderCode.create(gl, GL2ES2.GL_FRAGMENT_SHADER, 1, this.getClass(),
- * "shader", new String[] { "vertex" }, "shader/bin", "fragment");
+ * "shader", new String[] { "vertex" }, null,
+ * "shader/bin", "fragment", null, true);
* ShaderProgram sp0 = new ShaderProgram();
* sp0.add(gl, vp0, System.err);
* sp0.add(gl, fp0, System.err);
@@ -380,8 +462,12 @@ public class ShaderCode {
* @param context class used to help resolving the source and binary location
* @param srcRoot relative <i>root</i> path for <code>srcBasenames</code> optional
* @param srcBasenames basenames w/o path or suffix relative to <code>srcRoot</code> for the shader's source code
+ * @param srcSuffixOpt optional custom suffix for shader's source file,
+ * if {@code null} {@link #getFileSuffix(boolean, int)} is being used.
* @param binRoot relative <i>root</i> path for <code>binBasenames</code>
* @param binBasename basename w/o path or suffix relative to <code>binRoot</code> for the shader's binary code
+ * @param binSuffixOpt optional custom suffix for shader's binary file,
+ * if {@code null} {@link #getFileSuffix(boolean, int)} is being used.
* @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
* which can be edited later on at the costs of a String conversion when passing to
* {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
@@ -389,7 +475,7 @@ public class ShaderCode {
* which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
* at no additional costs.
*
- * @throws IllegalArgumentException if <code>count</count> and <code>srcBasenames.length</code> do not match
+ * @throws IllegalArgumentException if <code>count</code> and <code>srcBasenames.length</code> do not match
*
* @see #create(GL2ES2, int, int, Class, String[])
* @see #create(int, int, Class, int, String)
@@ -397,9 +483,12 @@ public class ShaderCode {
* @see #getFileSuffix(boolean, int)
* @see ShaderUtil#getShaderBinaryFormats(GL)
* @see #getBinarySubPath(int)
+ *
+ * @since 2.3.2
*/
public static ShaderCode create(final GL2ES2 gl, final int type, final int count, final Class<?> context,
- final String srcRoot, final String[] srcBasenames, final String binRoot, final String binBasename,
+ final String srcRoot, final String[] srcBasenames, final String srcSuffixOpt,
+ final String binRoot, final String binBasename, final String binSuffixOpt,
final boolean mutableStringBuilder) {
ShaderCode res = null;
final String srcPath[];
@@ -408,7 +497,7 @@ public class ShaderCode {
if( null!=srcBasenames && ShaderUtil.isShaderCompilerAvailable(gl) ) {
srcPath = new String[srcBasenames.length];
- final String srcSuffix = getFileSuffix(false, type);
+ final String srcSuffix = null != srcSuffixOpt ? srcSuffixOpt : getFileSuffix(false, type);
if( null != srcRoot && srcRoot.length() > 0 ) {
for(int i=0; i<srcPath.length; i++) {
srcPath[i] = srcRoot + '/' + srcBasenames[i] + "." + srcSuffix;
@@ -428,7 +517,7 @@ public class ShaderCode {
}
if( null!=binBasename ) {
final Set<Integer> binFmts = ShaderUtil.getShaderBinaryFormats(gl);
- final String binSuffix = getFileSuffix(true, type);
+ final String binSuffix = null != binSuffixOpt ? binSuffixOpt : getFileSuffix(true, type);
for(final Iterator<Integer> iter=binFmts.iterator(); iter.hasNext(); ) {
final int bFmt = iter.next().intValue();
final String bFmtPath = getBinarySubPath(bFmt);
@@ -445,7 +534,95 @@ public class ShaderCode {
}
/**
- * Simplified variation of {@link #create(GL2ES2, int, int, Class, String, String[], String, String, boolean)}.
+ * Simplified variation of {@link #create(GL2ES2, int, int, Class, String, String[], String, String, String, String, boolean)}.
+ * <p>
+ * Convenient creation method for instantiating a complete {@link ShaderCode} object
+ * either from source code using {@link #create(GL2ES2, int, int, Class, String[])},
+ * or from a binary code using {@link #create(int, int, Class, int, String)},
+ * whatever is available first.
+ * </p>
+ * <p>
+ * The source and binary location names are expected w/o suffixes which are
+ * resolved and appended using {@link #getFileSuffix(boolean, int)}.
+ * </p>
+ * <p>
+ * Additionally, the binary resource is expected within a subfolder of <code>binRoot</code>
+ * which reflects the vendor specific binary format, see {@link #getBinarySubPath(int)}.
+ * All {@link ShaderUtil#getShaderBinaryFormats(GL)} are being iterated
+ * using the binary subfolder, the first existing resource is being used.
+ * </p>
+ *
+ * Example:
+ * <pre>
+ * Your std JVM layout (plain or within a JAR):
+ *
+ * org/test/glsl/MyShaderTest.class
+ * org/test/glsl/shader/vertex.vp
+ * org/test/glsl/shader/fragment.fp
+ * org/test/glsl/shader/bin/nvidia/vertex.bvp
+ * org/test/glsl/shader/bin/nvidia/fragment.bfp
+ *
+ * Your Android APK layout:
+ *
+ * classes.dex
+ * assets/org/test/glsl/shader/vertex.vp
+ * assets/org/test/glsl/shader/fragment.fp
+ * assets/org/test/glsl/shader/bin/nvidia/vertex.bvp
+ * assets/org/test/glsl/shader/bin/nvidia/fragment.bfp
+ * ...
+ *
+ * Your invocation in org/test/glsl/MyShaderTest.java:
+ *
+ * ShaderCode vp0 = ShaderCode.create(gl, GL2ES2.GL_VERTEX_SHADER, 1, this.getClass(),
+ * "shader", new String[] { "vertex" }, "shader/bin", "vertex", true);
+ * ShaderCode fp0 = ShaderCode.create(gl, GL2ES2.GL_FRAGMENT_SHADER, 1, this.getClass(),
+ * "shader", new String[] { "vertex" }, "shader/bin", "fragment", true);
+ * ShaderProgram sp0 = new ShaderProgram();
+ * sp0.add(gl, vp0, System.err);
+ * sp0.add(gl, fp0, System.err);
+ * st.attachShaderProgram(gl, sp0, true);
+ * </pre>
+ * A simplified entry point is {@link #create(GL2ES2, int, Class, String, String, String, boolean)}.
+ *
+ * <p>
+ * The location is finally being resolved using the <code>context</code> class, see {@link #readShaderBinary(Class, String)}.
+ * </p>
+ *
+ * @param gl current GL object to determine whether a shader compiler is available (if <code>source</code> is used),
+ * or to determine the shader binary format (if <code>binary</code> is used).
+ * @param type either {@link GL2ES2#GL_VERTEX_SHADER}, {@link GL2ES2#GL_FRAGMENT_SHADER}, {@link GL3#GL_GEOMETRY_SHADER},
+ * {@link GL4#GL_TESS_CONTROL_SHADER} or {@link GL4#GL_TESS_EVALUATION_SHADER}.
+ * @param count number of shaders
+ * @param context class used to help resolving the source and binary location
+ * @param srcRoot relative <i>root</i> path for <code>srcBasenames</code> optional
+ * @param srcBasenames basenames w/o path or suffix relative to <code>srcRoot</code> for the shader's source code
+ * @param binRoot relative <i>root</i> path for <code>binBasenames</code>
+ * @param binBasename basename w/o path or suffix relative to <code>binRoot</code> for the shader's binary code
+ * @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
+ * which can be edited later on at the costs of a String conversion when passing to
+ * {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
+ * If <code>false</code> method returns an immutable <code>String</code> instance,
+ * which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
+ * at no additional costs.
+ *
+ * @throws IllegalArgumentException if <code>count</code> and <code>srcBasenames.length</code> do not match
+ *
+ * @see #create(GL2ES2, int, int, Class, String, String[], String, String, String, String, boolean)
+ * @see #readShaderSource(Class, String)
+ * @see #getFileSuffix(boolean, int)
+ * @see ShaderUtil#getShaderBinaryFormats(GL)
+ * @see #getBinarySubPath(int)
+ */
+ public static ShaderCode create(final GL2ES2 gl, final int type, final int count, final Class<?> context,
+ final String srcRoot, final String[] srcBasenames,
+ final String binRoot, final String binBasename,
+ final boolean mutableStringBuilder) {
+ return create(gl, type, count, context, srcRoot, srcBasenames, null,
+ binRoot, binBasename, null, mutableStringBuilder);
+ }
+
+ /**
+ * Simplified variation of {@link #create(GL2ES2, int, int, Class, String, String[], String, String, String, String, boolean)}.
* <p>
* Example:
* <pre>
@@ -469,9 +646,76 @@ public class ShaderCode {
* Your invocation in org/test/glsl/MyShaderTest.java:
*
* ShaderCode vp0 = ShaderCode.create(gl, GL2ES2.GL_VERTEX_SHADER, this.getClass(),
- * "shader", "shader/bin", "vertex");
+ * "shader", "shader/bin", "vertex", null, null, true);
* ShaderCode fp0 = ShaderCode.create(gl, GL2ES2.GL_FRAGMENT_SHADER, this.getClass(),
- * "shader", "shader/bin", "fragment");
+ * "shader", "shader/bin", "fragment", null, null, true);
+ * ShaderProgram sp0 = new ShaderProgram();
+ * sp0.add(gl, vp0, System.err);
+ * sp0.add(gl, fp0, System.err);
+ * st.attachShaderProgram(gl, sp0, true);
+ * </pre>
+ * </p>
+ *
+ * @param gl current GL object to determine whether a shader compiler is available (if <code>source</code> is used),
+ * or to determine the shader binary format (if <code>binary</code> is used).
+ * @param type either {@link GL2ES2#GL_VERTEX_SHADER}, {@link GL2ES2#GL_FRAGMENT_SHADER}, {@link GL3#GL_GEOMETRY_SHADER},
+ * {@link GL4#GL_TESS_CONTROL_SHADER} or {@link GL4#GL_TESS_EVALUATION_SHADER}.
+ * @param context class used to help resolving the source and binary location
+ * @param srcRoot relative <i>root</i> path for <code>basename</code> optional
+ * @param binRoot relative <i>root</i> path for <code>basename</code>
+ * @param basename basename w/o path or suffix relative to <code>srcRoot</code> and <code>binRoot</code>
+ * for the shader's source and binary code.
+ * @param srcSuffixOpt optional custom suffix for shader's source file,
+ * if {@code null} {@link #getFileSuffix(boolean, int)} is being used.
+ * @param binSuffixOpt optional custom suffix for shader's binary file,
+ * if {@code null} {@link #getFileSuffix(boolean, int)} is being used.
+ * @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
+ * which can be edited later on at the costs of a String conversion when passing to
+ * {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
+ * If <code>false</code> method returns an immutable <code>String</code> instance,
+ * which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
+ * at no additional costs.
+ * @throws IllegalArgumentException if <code>count</code> is not 1
+ *
+ * @see #create(GL2ES2, int, int, Class, String, String[], String, String, String, String, boolean)
+ * @since 2.3.2
+ */
+ public static ShaderCode create(final GL2ES2 gl, final int type, final Class<?> context,
+ final String srcRoot, final String binRoot,
+ final String basename, final String srcSuffixOpt, final String binSuffixOpt,
+ final boolean mutableStringBuilder) {
+ return create(gl, type, 1, context, srcRoot, new String[] { basename }, srcSuffixOpt,
+ binRoot, basename, binSuffixOpt, mutableStringBuilder );
+ }
+
+ /**
+ * Simplified variation of {@link #create(GL2ES2, int, Class, String, String, String, String, String, boolean)}.
+ * <p>
+ * Example:
+ * <pre>
+ * Your std JVM layout (plain or within a JAR):
+ *
+ * org/test/glsl/MyShaderTest.class
+ * org/test/glsl/shader/vertex.vp
+ * org/test/glsl/shader/fragment.fp
+ * org/test/glsl/shader/bin/nvidia/vertex.bvp
+ * org/test/glsl/shader/bin/nvidia/fragment.bfp
+ *
+ * Your Android APK layout:
+ *
+ * classes.dex
+ * assets/org/test/glsl/shader/vertex.vp
+ * assets/org/test/glsl/shader/fragment.fp
+ * assets/org/test/glsl/shader/bin/nvidia/vertex.bvp
+ * assets/org/test/glsl/shader/bin/nvidia/fragment.bfp
+ * ...
+ *
+ * Your invocation in org/test/glsl/MyShaderTest.java:
+ *
+ * ShaderCode vp0 = ShaderCode.create(gl, GL2ES2.GL_VERTEX_SHADER, this.getClass(),
+ * "shader", "shader/bin", "vertex", true);
+ * ShaderCode fp0 = ShaderCode.create(gl, GL2ES2.GL_FRAGMENT_SHADER, this.getClass(),
+ * "shader", "shader/bin", "fragment", true);
* ShaderProgram sp0 = new ShaderProgram();
* sp0.add(gl, vp0, System.err);
* sp0.add(gl, fp0, System.err);
@@ -495,13 +739,12 @@ public class ShaderCode {
* If <code>false</code> method returns an immutable <code>String</code> instance,
* which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
* at no additional costs.
- * @throws IllegalArgumentException if <code>count</count> is not 1
- *
- * @see #create(GL2ES2, int, int, Class, String, String[], String, String)
+ * @throws IllegalArgumentException if <code>count</code> is not 1
*/
public static ShaderCode create(final GL2ES2 gl, final int type, final Class<?> context,
- final String srcRoot, final String binRoot, final String basename, final boolean mutableStringBuilder) {
- return create(gl, type, 1, context, srcRoot, new String[] { basename }, binRoot, basename, mutableStringBuilder );
+ final String srcRoot, final String binRoot, final String basename,
+ final boolean mutableStringBuilder) {
+ return ShaderCode.create(gl, type, context, srcRoot, binRoot, basename, null, null, mutableStringBuilder);
}
/**
@@ -815,7 +1058,6 @@ public class ShaderCode {
}
}
- @SuppressWarnings("resource")
private static int readShaderSource(final Class<?> context, final URLConnection conn, final StringBuilder result, int lineno) throws IOException {
if(DEBUG_CODE) {
if(0 == lineno) {
@@ -858,10 +1100,11 @@ public class ShaderCode {
}
/**
+ * Reads shader source located in <code>conn</code>.
*
- * @param context
- * @param conn
- * @param result
+ * @param context class used to help resolve the source location, may be {@code null}
+ * @param conn the {@link URLConnection} of the shader source
+ * @param result {@link StringBuilder} sink for the resulting shader source code
* @throws IOException
*/
public static void readShaderSource(final Class<?> context, final URLConnection conn, final StringBuilder result) throws IOException {
@@ -899,6 +1142,28 @@ public class ShaderCode {
}
/**
+ * Reads shader source located from {@link Uri#absolute} {@link Uri} <code>sourceLocation</code>.
+ * @param sourceLocation {@link Uri} location of shader source
+ * @param mutableStringBuilder if <code>true</code> method returns a mutable <code>StringBuilder</code> instance
+ * which can be edited later on at the costs of a String conversion when passing to
+ * {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}.
+ * If <code>false</code> method returns an immutable <code>String</code> instance,
+ * which can be passed to {@link GL2ES2#glShaderSource(int, int, String[], IntBuffer)}
+ * at no additional costs.
+ * @throws IOException
+ * @since 2.3.2
+ */
+ public static CharSequence readShaderSource(final Uri sourceLocation, final boolean mutableStringBuilder) throws IOException {
+ final URLConnection conn = IOUtil.openURL(sourceLocation.toURL(), "ShaderCode ");
+ if (conn == null) {
+ return null;
+ }
+ final StringBuilder result = new StringBuilder();
+ readShaderSource(null, conn, result);
+ return mutableStringBuilder ? result : result.toString();
+ }
+
+ /**
* Reads shader binary located in <code>path</code>,
* either relative to the <code>context</code> class or absolute <i>as-is</i>.
* <p>
@@ -925,6 +1190,25 @@ public class ShaderCode {
}
}
+ /**
+ * Reads shader binary located from {@link Uri#absolute} {@link Uri} <code>binLocation</code>.
+ * @param binLocation {@link Uri} location of shader binary
+ * @throws IOException
+ * @since 2.3.2
+ */
+ public static ByteBuffer readShaderBinary(final Uri binLocation) throws IOException {
+ final URLConnection conn = IOUtil.openURL(binLocation.toURL(), "ShaderCode ");
+ if (conn == null) {
+ return null;
+ }
+ final BufferedInputStream bis = new BufferedInputStream( conn.getInputStream() );
+ try {
+ return IOUtil.copyStream2ByteBuffer( bis );
+ } finally {
+ IOUtil.close(bis, false);
+ }
+ }
+
// Shall we use: #ifdef GL_FRAGMENT_PRECISION_HIGH .. #endif for using highp in fragment shader if avail ?
/** Default precision of {@link GL#isGLES2() ES2} for {@link GL2ES2#GL_VERTEX_SHADER vertex-shader}: {@value #es2_default_precision_vp} */
public static final String es2_default_precision_vp = "\nprecision highp float;\nprecision highp int;\n/*precision lowp sampler2D;*/\n/*precision lowp samplerCube;*/\n";
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/EyeParameter.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/EyeParameter.java
index 075da340b..43a6cfc58 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/EyeParameter.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/EyeParameter.java
@@ -30,7 +30,7 @@ package com.jogamp.opengl.util.stereo;
import com.jogamp.opengl.math.FovHVHalves;
/**
- * Constant parameter for one eye.
+ * Constant single eye parameter of the viewer, relative to its {@link ViewerPose}.
*/
public final class EyeParameter {
/** Eye number, <code>0</code> for the left eye and <code>1</code> for the right eye. */
@@ -39,7 +39,7 @@ public final class EyeParameter {
/** float[3] eye position vector used to define eye height in meter relative to <i>actor</i>. */
public final float[] positionOffset;
- /** Field of view in both directions, may not be centered, either in radians or tangent. */
+ /** Field of view in both directions, may not be centered, either {@link FovHVHalves#inTangents} or radians. */
public final FovHVHalves fovhv;
/** IPD related horizontal distance from nose to pupil in meter. */
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/LocationSensorParameter.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/LocationSensorParameter.java
new file mode 100644
index 000000000..b795927cd
--- /dev/null
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/LocationSensorParameter.java
@@ -0,0 +1,51 @@
+/**
+ * Copyright 2015 JogAmp Community. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without modification, are
+ * permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this list of
+ * conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright notice, this list
+ * of conditions and the following disclaimer in the documentation and/or other materials
+ * provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY JogAmp Community ``AS IS'' AND ANY EXPRESS OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+ * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JogAmp Community OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+ * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * The views and conclusions contained in the software and documentation are those of the
+ * authors and should not be interpreted as representing official policies, either expressed
+ * or implied, of JogAmp Community.
+ */
+package com.jogamp.opengl.util.stereo;
+
+import com.jogamp.opengl.math.geom.Frustum;
+
+/**
+ * Constant parameter of the positioning sensor to locate the {@link ViewerPose}.
+ */
+public final class LocationSensorParameter {
+ /** The {@link Frustum} of the location sensor. */
+ public final Frustum frustum;
+ /** The {@link Frustum}'s {@link Frustum.FovDesc} description of the location sensor. */
+ public final Frustum.FovDesc frustumDesc;
+ /** The {@link Frustum}'s float[16] projection matrix of the location sensor. */
+ public final float[] frustumProjMat;
+
+ public LocationSensorParameter(final Frustum.FovDesc fovDesc) {
+ this.frustumDesc = fovDesc;
+ this.frustum = new Frustum();
+ this.frustumProjMat = frustum.updateByFovDesc(new float[16], 0, true, fovDesc);
+ }
+ public final String toString() {
+ return "LocationSensor["+frustumDesc+"]";
+ }
+} \ No newline at end of file
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoClientRenderer.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoClientRenderer.java
index b1a38ab06..8ecd8b8f7 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoClientRenderer.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoClientRenderer.java
@@ -189,10 +189,7 @@ public class StereoClientRenderer implements GLEventListener {
final int[] eyeOrder = deviceRenderer.getDevice().getEyeRenderOrder();
final int eyeCount = eyeOrder.length;
- // Update eye pos upfront to have same (almost) results
- for(int eyeNum=0; eyeNum<eyeCount; eyeNum++) {
- deviceRenderer.updateEyePose(eyeNum);
- }
+ final ViewerPose viewerPose = deviceRenderer.updateViewerPose();
if( 1 == fboCount ) {
fbos[0].bind(gl);
@@ -213,7 +210,7 @@ public class StereoClientRenderer implements GLEventListener {
public void run(final GLAutoDrawable drawable, final GLEventListener listener) {
final StereoGLEventListener sl = (StereoGLEventListener) listener;
sl.reshapeForEye(drawable, viewport.getX(), viewport.getY(), viewport.getWidth(), viewport.getHeight(),
- eye.getEyeParameter(), eye.getLastEyePose());
+ eye.getEyeParameter(), viewerPose);
sl.display(drawable, displayFlags);
} };
helper.runForAllGLEventListener(drawable, reshapeDisplayAction);
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceConfig.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceConfig.java
index 04fd8343c..44129de02 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceConfig.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceConfig.java
@@ -37,6 +37,7 @@ import com.jogamp.opengl.util.stereo.EyeParameter;
import com.jogamp.opengl.util.stereo.StereoDeviceConfig;
import com.jogamp.opengl.util.stereo.StereoDeviceRenderer;
import com.jogamp.opengl.util.stereo.StereoUtil;
+import com.jogamp.opengl.util.stereo.StereoDevice;
/**
* Configuration for {@link GenericStereoDevice}s.
@@ -54,6 +55,7 @@ public class GenericStereoDeviceConfig extends StereoDeviceConfig {
final float interpupillaryDistanceInMeters,
final int[] eyeRenderOrder,
final EyeParameter[] defaultEyeParam,
+ final int supportedSensorBits,
final DistortionMesh.Producer distortionMeshProducer,
final int supportedDistortionBits,
final int recommendedDistortionBits,
@@ -71,6 +73,7 @@ public class GenericStereoDeviceConfig extends StereoDeviceConfig {
this.interpupillaryDistanceInMeters = interpupillaryDistanceInMeters;
this.eyeRenderOrder = eyeRenderOrder;
this.defaultEyeParam = defaultEyeParam;
+ this.supportedSensorBits = supportedSensorBits;
this.distortionMeshProducer = distortionMeshProducer;
this.supportedDistortionBits = supportedDistortionBits;
this.recommendedDistortionBits = recommendedDistortionBits;
@@ -92,6 +95,7 @@ public class GenericStereoDeviceConfig extends StereoDeviceConfig {
this.interpupillaryDistanceInMeters = source.interpupillaryDistanceInMeters;
this.eyeRenderOrder = source.eyeRenderOrder;
this.defaultEyeParam = source.defaultEyeParam;
+ this.supportedSensorBits = source.supportedSensorBits;
this.distortionMeshProducer = source.distortionMeshProducer;
this.supportedDistortionBits = source.supportedDistortionBits;
this.recommendedDistortionBits = source.recommendedDistortionBits;
@@ -141,7 +145,8 @@ public class GenericStereoDeviceConfig extends StereoDeviceConfig {
" [m], eyeParam "+Arrays.toString(defaultEyeParam)+
", distortionBits[supported ["+StereoUtil.distortionBitsToString(supportedDistortionBits)+
"], recommended ["+StereoUtil.distortionBitsToString(recommendedDistortionBits)+
- "], minimum ["+StereoUtil.distortionBitsToString(minimumDistortionBits)+"]]]";
+ "], minimum ["+StereoUtil.distortionBitsToString(minimumDistortionBits)+"]]"+
+ ", sensorBits[supported ["+StereoUtil.sensorBitsToString(supportedSensorBits)+"]]]";
}
/** Configuration Name */
@@ -164,9 +169,13 @@ public class GenericStereoDeviceConfig extends StereoDeviceConfig {
public final float[/*per-eye*/][/*xy*/] pupilCenterFromTopLeft;
public final int[] eyeRenderOrder;
public final EyeParameter[] defaultEyeParam;
+
+ /** Supported sensor bits, see {@link StereoDevice#SENSOR_ORIENTATION}. */
+ public final int supportedSensorBits;
+
public final DistortionMesh.Producer distortionMeshProducer;
- /** Supported distortion bits, see {@link StereoDeviceRenderer.DISTORTION_BARREL}. */
+ /** Supported distortion bits, see {@link StereoDeviceRenderer#DISTORTION_BARREL}. */
public final int supportedDistortionBits;
/** Recommended distortion bits, see {@link StereoDeviceRenderer.DISTORTION_BARREL}. */
public final int recommendedDistortionBits;
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceFactory.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceFactory.java
index 6adb96ba4..957758e78 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceFactory.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/generic/GenericStereoDeviceFactory.java
@@ -70,6 +70,7 @@ public class GenericStereoDeviceFactory extends StereoDeviceFactory {
// degrees: 45/2 l, 45/2 r, 45/2 * aspect t, 45/2 * aspect b
FovHVHalves.byFovyRadianAndAspect(45f*d2r, 1280f / 800f),
0f /* distNoseToPupil */, 0f /* verticalDelta */, 0f /* eyeReliefInMeters */) },
+ 0, // supported sensor bits
null, // mash producer distortion bits
0, // supported distortion bits
0, // recommended distortion bits
@@ -114,6 +115,7 @@ public class GenericStereoDeviceFactory extends StereoDeviceFactory {
interpupillaryDistanceInMeters/2f /* distNoseToPupil */, 0f /* verticalDelta */, 0.010f /* eyeReliefInMeters */),
new EyeParameter(1, defaultEyePositionOffset, defaultSBSEyeFovRight,
-interpupillaryDistanceInMeters/2f /* distNoseToPupil */, 0f /* verticalDelta */, 0.010f /* eyeReliefInMeters */) },
+ 0, // supported sensor bits
null, // mash producer distortion bits
0, // supported distortion bits
0, // recommended distortion bits
@@ -171,6 +173,7 @@ public class GenericStereoDeviceFactory extends StereoDeviceFactory {
interpupillaryDistanceInMeters/2f /* distNoseToPupil */, 0f /* verticalDelta */, 0.010f /* eyeReliefInMeters */),
new EyeParameter(1, defaultEyePositionOffset, defaultSBSEyeFovRight,
-interpupillaryDistanceInMeters/2f /* distNoseToPupil */, 0f /* verticalDelta */, 0.010f /* eyeReliefInMeters */) },
+ 0, // supported sensor bits
lenseDistMeshProduce,
// supported distortion bits
StereoDeviceRenderer.DISTORTION_BARREL | StereoDeviceRenderer.DISTORTION_CHROMATIC | StereoDeviceRenderer.DISTORTION_VIGNETTE,
diff --git a/src/jogl/classes/jogamp/opengl/ProjectFloat.java b/src/jogl/classes/jogamp/opengl/ProjectFloat.java
index ee9560214..5ec5a8e3f 100644
--- a/src/jogl/classes/jogamp/opengl/ProjectFloat.java
+++ b/src/jogl/classes/jogamp/opengl/ProjectFloat.java
@@ -170,7 +170,8 @@ public class ProjectFloat {
* @param aspect
* @param zNear
* @param zFar
- * @throws GLException with GL_INVALID_VALUE if zNear is <= 0, or zFar < 0, or if zNear == zFar.
+ * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+ * @see FloatUtil#makePerspective(float[], int, boolean, float, float, float, float)
*/
public void gluPerspective(final GLMatrixFunc gl, final float fovy_deg, final float aspect, final float zNear, final float zFar) throws GLException {
gl.glMultMatrixf(FloatUtil.makePerspective(mat4Tmp1, 0, true, fovy_deg * FloatUtil.PI / 180.0f, aspect, zNear, zFar), 0);
diff --git a/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDevice.java b/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDevice.java
index 9c0a5045b..bfe93b59c 100644
--- a/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDevice.java
+++ b/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDevice.java
@@ -36,9 +36,11 @@ import com.jogamp.nativewindow.util.RectangleImmutable;
import com.jogamp.opengl.math.FovHVHalves;
import com.jogamp.opengl.util.stereo.StereoDeviceConfig;
import com.jogamp.opengl.util.stereo.EyeParameter;
+import com.jogamp.opengl.util.stereo.LocationSensorParameter;
import com.jogamp.opengl.util.stereo.StereoDevice;
import com.jogamp.opengl.util.stereo.StereoDeviceFactory;
import com.jogamp.opengl.util.stereo.StereoDeviceRenderer;
+import com.jogamp.opengl.util.stereo.StereoUtil;
import com.jogamp.opengl.util.stereo.generic.GenericStereoDeviceConfig;
import com.jogamp.opengl.util.stereo.generic.GenericStereoDeviceFactory;
@@ -117,6 +119,7 @@ public class GenericStereoDevice implements StereoDevice {
public final Point surfacePos;
private final FovHVHalves[] defaultEyeFov;
+ private int usedSensorBits;
private boolean sensorsStarted = false;
public GenericStereoDevice(final StereoDeviceFactory factory, final int deviceIndex, final StereoDeviceConfig customConfig) {
@@ -137,6 +140,9 @@ public class GenericStereoDevice implements StereoDevice {
for(int i=0; i<defaultEyeFov.length; i++) {
defaultEyeFov[i] = config.defaultEyeParam[i].fovhv;
}
+
+ // default
+ usedSensorBits = 0;
}
@Override
@@ -144,7 +150,8 @@ public class GenericStereoDevice implements StereoDevice {
@Override
public String toString() {
- return "GenericStereoDevice["+config+", surfacePos "+surfacePos+"]";
+ return "GenericStereoDevice["+config+", surfacePos "+surfacePos+
+ ", sensorBits[enabled ["+StereoUtil.sensorBitsToString(getEnabledSensorBits())+"]]]";
}
public void setSurfacePosition(final int x, final int y) {
@@ -153,52 +160,69 @@ public class GenericStereoDevice implements StereoDevice {
@Override
public final void dispose() {
- // NOP
+ stopSensors();
}
@Override
- public boolean isValid() {
- return true;
- }
+ public boolean isValid() { return true; }
@Override
- public final PointImmutable getPosition() {
- return surfacePos;
- }
+ public final PointImmutable getPosition() { return surfacePos; }
@Override
- public final DimensionImmutable getSurfaceSize() {
- return config.surfaceSizeInPixels;
- }
+ public final DimensionImmutable getSurfaceSize() { return config.surfaceSizeInPixels; }
@Override
public int getRequiredRotation() { return 0; }
@Override
- public float[] getDefaultEyePositionOffset() {
- return config.defaultEyeParam[0].positionOffset;
- }
+ public float[] getDefaultEyePositionOffset() { return config.defaultEyeParam[0].positionOffset; }
@Override
- public final FovHVHalves[] getDefaultFOV() {
- return defaultEyeFov;
- }
+ public final FovHVHalves[] getDefaultFOV() { return defaultEyeFov; }
+
+ @Override
+ public final LocationSensorParameter getLocationSensorParams() { return null; }
@Override
- public final boolean startSensors(final boolean start) {
- if( start && !sensorsStarted ) {
- if( startSensorsImpl(true) ) {
+ public final void resetLocationSensorOrigin() { }
+
+ @Override
+ public final boolean startSensors(final int desiredSensorBits, final int requiredSensorBits) {
+ if( !sensorsStarted ) {
+ if( requiredSensorBits != ( config.supportedSensorBits & requiredSensorBits ) ) {
+ // required sensors not available
+ return false;
+ }
+ if( 0 == ( config.supportedSensorBits & ( requiredSensorBits | desiredSensorBits ) ) ) {
+ // no sensors available
+ return false;
+ }
+ if( startSensorsImpl(true, desiredSensorBits, requiredSensorBits) ) {
sensorsStarted = true;
return true;
} else {
- sensorsStarted = false;
return false;
}
- } else if( sensorsStarted ) {
- if( startSensorsImpl(false) ) {
+ } else {
+ // No state change -> Success
+ return true;
+ }
+ }
+ protected boolean startSensorsImpl(final boolean start, final int desiredSensorBits, final int requiredSensorBits) {
+ // TODO: Add SPI for sensors
+ // TODO: start sensors in override / or SPI
+ // TODO: set usedSensorBits
+ return false;
+ }
+
+ @Override
+ public final boolean stopSensors() {
+ if( sensorsStarted ) {
+ if( startSensorsImpl(false, 0, 0) ) {
sensorsStarted = false;
+ usedSensorBits = 0;
return true;
} else {
- sensorsStarted = true;
return false;
}
} else {
@@ -206,12 +230,21 @@ public class GenericStereoDevice implements StereoDevice {
return true;
}
}
- private boolean startSensorsImpl(final boolean start) { return start; }
@Override
public boolean getSensorsStarted() { return sensorsStarted; }
@Override
+ public final int getSupportedSensorBits() {
+ return config.supportedSensorBits;
+ }
+
+ @Override
+ public final int getEnabledSensorBits() {
+ return usedSensorBits;
+ }
+
+ @Override
public int[] getEyeRenderOrder() { return config.eyeRenderOrder; }
@Override
diff --git a/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDeviceRenderer.java b/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDeviceRenderer.java
index 6650342e7..21567a0f1 100644
--- a/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDeviceRenderer.java
+++ b/src/jogl/classes/jogamp/opengl/util/stereo/GenericStereoDeviceRenderer.java
@@ -49,7 +49,7 @@ import com.jogamp.opengl.util.GLArrayDataServer;
import com.jogamp.opengl.util.glsl.ShaderCode;
import com.jogamp.opengl.util.glsl.ShaderProgram;
import com.jogamp.opengl.util.stereo.EyeParameter;
-import com.jogamp.opengl.util.stereo.EyePose;
+import com.jogamp.opengl.util.stereo.ViewerPose;
import com.jogamp.opengl.util.stereo.StereoDevice;
import com.jogamp.opengl.util.stereo.StereoDeviceRenderer;
import com.jogamp.opengl.util.stereo.StereoUtil;
@@ -82,7 +82,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
private final EyeParameter eyeParameter;
- private final EyePose eyePose;
@Override
public final RectangleImmutable getViewport() { return viewport; }
@@ -90,12 +89,9 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
@Override
public final EyeParameter getEyeParameter() { return eyeParameter; }
- @Override
- public final EyePose getLastEyePose() { return eyePose; }
-
- private GenericEye(final GenericStereoDevice device, final int distortionBits,
- final float[] eyePositionOffset, final EyeParameter eyeParam,
- final DimensionImmutable textureSize, final RectangleImmutable eyeViewport) {
+ /* pp */ GenericEye(final GenericStereoDevice device, final int distortionBits,
+ final float[] eyePositionOffset, final EyeParameter eyeParam,
+ final DimensionImmutable textureSize, final RectangleImmutable eyeViewport) {
this.eyeName = eyeParam.number;
this.distortionBits = distortionBits;
this.viewport = eyeViewport;
@@ -123,10 +119,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
this.eyeParameter = eyeParam;
- this.eyePose = new EyePose(eyeName);
-
- updateEyePose(device); // 1st init
-
// Setup: eyeToSourceUVScale, eyeToSourceUVOffset
if( usePP ) {
final ScaleAndOffset2D textureScaleAndOffset = new ScaleAndOffset2D(eyeParam.fovhv, textureSize, eyeViewport);
@@ -333,18 +325,18 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
indices.enableBuffer(gl, false);
}
- private void dispose(final GL2ES2 gl) {
+ /* pp */ void dispose(final GL2ES2 gl) {
if( null == iVBO ) return;
iVBO.destroy(gl);
indices.destroy(gl);
}
- private void enableVBO(final GL2ES2 gl, final boolean enable) {
+ /* pp */ void enableVBO(final GL2ES2 gl, final boolean enable) {
if( null == iVBO ) return;
iVBO.enableBuffer(gl, enable);
indices.bindBuffer(gl, enable); // keeps VBO binding if enable:=true
}
- private void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
+ /* pp */ void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
if( null == iVBO ) return;
gl.glUniform(eyeToSourceUVScale);
gl.glUniform(eyeToSourceUVOffset);
@@ -354,15 +346,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
}
}
- /**
- * Updates {@link #ovrEyePose} and it's extracted
- * {@link #eyeRenderPoseOrientation} and {@link #eyeRenderPosePosition}.
- * @param hmdCtx used get the {@link #ovrEyePose} via {@link OVR#ovrHmd_GetEyePose(OvrHmdContext, int)}
- */
- private EyePose updateEyePose(final GenericStereoDevice hmdCtx) {
- return eyePose;
- }
-
@Override
public String toString() {
final String ppTxt = null == iVBO ? ", no post-processing" :
@@ -373,12 +356,13 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
", "+eyeParameter+
", vertices "+vertexCount+", indices "+indexCount+
ppTxt+
- ", desc"+eyeParameter+", "+eyePose+"]";
+ ", desc "+eyeParameter+"]";
}
}
private final GenericStereoDevice device;
private final GenericEye[] eyes;
+ private final ViewerPose viewerPose;
private final int distortionBits;
private final int textureCount;
private final DimensionImmutable[] eyeTextureSizes;
@@ -445,9 +429,11 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
this.totalTextureSize = zeroSize;
texUnit0 = null;
}
+ viewerPose = new ViewerPose();
for(int i=0; i<eyeParam.length; i++) {
eyes[i] = new GenericEye(context, this.distortionBits, eyePositionOffset, eyeParam[i], textureSizes[i], eyeViewports[i]);
}
+
sp = null;
}
@@ -559,8 +545,14 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
}
@Override
- public final EyePose updateEyePose(final int eyeNum) {
- return eyes[eyeNum].updateEyePose(device);
+ public final ViewerPose updateViewerPose() {
+ // NOP
+ return viewerPose;
+ }
+
+ @Override
+ public final ViewerPose getLastViewerPose() {
+ return viewerPose;
}
@Override
diff --git a/src/jogl/native/openmax/jogamp_opengl_util_av_impl_OMXGLMediaPlayer.c b/src/jogl/native/openmax/jogamp_opengl_util_av_impl_OMXGLMediaPlayer.c
index 3166306ba..fc2fb6ea3 100644
--- a/src/jogl/native/openmax/jogamp_opengl_util_av_impl_OMXGLMediaPlayer.c
+++ b/src/jogl/native/openmax/jogamp_opengl_util_av_impl_OMXGLMediaPlayer.c
@@ -39,7 +39,7 @@ JNIEXPORT jboolean JNICALL Java_jogamp_opengl_util_av_impl_OMXGLMediaPlayer_init
jclass c;
if (omxGLMediaPlayerClazz != NULL) {
- return;
+ return JNI_FALSE;
}
c = (*env)->FindClass(env, ClazzNameOMXGLMediaPlayer);
diff --git a/src/jogl/native/openmax/omx_tool.h b/src/jogl/native/openmax/omx_tool.h
index 022c56cf9..31abfc5b2 100644
--- a/src/jogl/native/openmax/omx_tool.h
+++ b/src/jogl/native/openmax/omx_tool.h
@@ -27,6 +27,7 @@
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
+#include <unistd.h>
// FIXME: Move all sync/buffer handling to Java - Already done -> GLMediaPlayerImpl
#define EGLIMAGE_MAX_BUFFERS 4
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDevice.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDevice.java
index d59863530..b6112650a 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDevice.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDevice.java
@@ -41,10 +41,30 @@ public interface StereoDevice {
public static final boolean DEBUG = Debug.debug("StereoDevice");
public static final boolean DUMP_DATA = Debug.isPropertyDefined("jogl.debug.StereoDevice.DumpData", true);
+ /**
+ * Sensor Bit: Orientation tracking
+ */
+ public static final int SENSOR_ORIENTATION = 1 << 0;
+
+ /**
+ * Sensor Bit: Yaw correction
+ */
+ public static final int SENSOR_YAW_CORRECTION = 1 << 1;
+
+ /**
+ * Sensor Bit: Positional tracking
+ */
+ public static final int SENSOR_POSITION = 1 << 2;
+
/** Return the factory used to create this device. */
public StereoDeviceFactory getFactory();
/** Disposes this {@link StereoDevice}, if {@link #isValid() valid}. */
public void dispose();
/**
@@ -96,13 +116,76 @@ public interface StereoDevice {
*/
public FovHVHalves[] getDefaultFOV();
- /** Start or stop sensors. Returns true if action was successful, otherwise false. */
- public boolean startSensors(boolean start);
+ /**
+ * Returns the {@link LocationSensorParameter} of the device
+ * if {@link #SENSOR_POSITION} is {@link #getSupportedSensorBits() supported},
+ * otherwise returns {@code null}.
+ */
+ public LocationSensorParameter getLocationSensorParams();
- /** Return true if sensors have been started, false otherwise */
+
+ /**
+ * Sets the location sensor's origin of this device to the current position.
+ * <p>
+ * In case {@link #SENSOR_POSITION} is not {@link #getSupportedSensorBits() supported},
+ * this method is a no-op.
+ * </p>
+ */
+ public void resetLocationSensorOrigin();
+
+ /**
+ * Start desired and required sensors. Returns true if action was successful, otherwise false.
+ * <p>
+ * Method fails if required sensors are not {@link #getSupportedSensorBits() supported}.
+ * </p>
+ * @param desiredSensorBits the desired optional sensors
+ * @param requiredSensorBits the required sensors
+ * @see #stopSensors()
+ * @see #getSensorsStarted()
+ * @see #getSupportedSensorBits()
+ * @see #getEnabledSensorBits()
+ */
+ public boolean startSensors(int desiredSensorBits, int requiredSensorBits);
+
+ /**
+ * Stop sensors. Returns true if action was successful, otherwise false.
+ * @see #startSensors(int, int)
+ * @see #getSensorsStarted()
+ * @see #getSupportedSensorBits()
+ * @see #getEnabledSensorBits()
+ */
+ public boolean stopSensors();
+
+ /**
+ * Return true if sensors have been started, false otherwise.
+ * @see #startSensors(int, int)
+ * @see #stopSensors()
+ * @see #getSupportedSensorBits()
+ * @see #getEnabledSensorBits()
+ */
public boolean getSensorsStarted();
/**
+ * Returns the supported sensor capability bits, e.g. {@link #SENSOR_ORIENTATION}, {@link #SENSOR_POSITION}
+ * of the {@link StereoDevice}.
+ * @see #startSensors(int, int)
+ * @see #stopSensors()
+ * @see #getSensorsStarted()
+ * @see #getEnabledSensorBits()
+ */
+ public int getSupportedSensorBits();
+
+ /**
+ * Returns the actual used sensor capability bits, e.g. {@link #SENSOR_ORIENTATION}, {@link #SENSOR_POSITION}
+ * in case the {@link #getSupportedSensorBits() device supports} them and if they are enabled.
+ * @see #startSensors(int, int)
+ * @see #stopSensors()
+ * @see #getSensorsStarted()
+ * @see #getSupportedSensorBits()
+ */
+ public int getEnabledSensorBits();
+
+ /**
* Returns an array of the preferred eye rendering order.
* The array length reflects the supported eye count.
* <p>
@@ -112,7 +195,7 @@ public interface StereoDevice {
public int[] getEyeRenderOrder();
/**
- * Returns the supported distortion compensation by the {@link StereoDeviceRenderer},
+ * Returns the supported distortion compensation of the {@link StereoDeviceRenderer},
* e.g. {@link StereoDeviceRenderer#DISTORTION_BARREL}, {@link StereoDeviceRenderer#DISTORTION_CHROMATIC}, etc.
* @see StereoDeviceRenderer#getDistortionBits()
* @see #createRenderer(int, int, float[], FovHVHalves[], float, int)
@@ -41,10 +41,30 @@ public interface StereoDevice {
/** Return the factory used to create this device. */
public StereoDeviceFactory getFactory();
- /** Disposes this {@link StereoDevice}, if {@link #isValid() valid}. */
+ /**
+ * Disposes this {@link StereoDevice}, if {@link #isValid() valid}.
+ * <p>
+ * Implementation shall {@link #stopSensors() stop sensors} and free all resources.
+ * </p>
+ */
public void dispose();
/**
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDeviceRenderer.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDeviceRenderer.java
index 2078a00a2..0d6539634 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDeviceRenderer.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoDeviceRenderer.java
@@ -42,10 +42,10 @@ import com.jogamp.opengl.math.FovHVHalves;
* <ul>
* <li>device.{@link #beginFrame(GL)}</li>
* <li>For both eyes:<ul>
- * <li>device.{@link #updateEyePose(int)}</li>
+ * <li>device.{@link #updateViewerPose(int)}</li>
* <li>if device.{@link #ppAvailable()}: Set the render target, e.g. FBO</li>
* <li>Set the viewport using {@link Eye#getViewport()}</li>
* <li>{@link StereoGLEventListener#reshapeForEye(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, EyePose) upstream.reshapeEye(..)}</li>
* <li>{@link StereoGLEventListener#display(com.jogamp.opengl.GLAutoDrawable, int) upstream.display(..)}.</li>
* </ul></li>
* <li>Reset the viewport</li>
@@ -89,7 +89,7 @@ public interface StereoDeviceRenderer {
/**
* Distortion Bit: Timewarp distortion technique to predict
* {@link EyePose} movement to reduce latency.
* <p>
* FIXME: Explanation needs refinement!
* </p>
@@ -113,10 +113,6 @@ public interface StereoDeviceRenderer {
* Returns the {@link EyeParameter} of this eye.
*/
public EyeParameter getEyeParameter();
- /**
- * Returns the last {@link EyePose} of this eye.
- */
- public EyePose getLastEyePose();
}
/**
@@ -125,10 +121,14 @@ public interface StereoDeviceRenderer {
public Eye getEye(final int eyeNum);
/**
- * Updates the {@link Eye#getLastEyePose()}
- * for the denoted <code>eyeNum</code>.
+ * Updates the {@link ViewerPose} and returns it.
+ */
+ public ViewerPose updateViewerPose();
+
+ /**
+ * Returns the last {@link ViewerPose}.
*/
- public EyePose updateEyePose(final int eyeNum);
+ public ViewerPose getLastViewerPose();
/**
* Returns used distortion compensation bits, e.g. {@link #DISTORTION_BARREL},
@@ -219,7 +219,7 @@ public interface StereoDeviceRenderer {
/**
* Begin stereoscopic post-processing, see {@link #ppAvailable()}.
* <p>
* {@link #updateEyePose(int)} for both eyes must be called upfront
* when rendering upstream {@link StereoGLEventListener}.
* </p>
*
@@ -42,10 +42,10 @@ import com.jogamp.opengl.math.FovHVHalves;
* <li>if device.{@link #ppAvailable()}: Set the render target, e.g. FBO</li>
* <li>Set the viewport using {@link Eye#getViewport()}</li>
- * <li>{@link StereoGLEventListener#reshapeForEye(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, EyePose) upstream.reshapeEye(..)}</li>
+ * <li>{@link StereoGLEventListener#reshapeForEye(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, ViewerPose) upstream.reshapeEye(..)}</li>
* <li>{@link StereoGLEventListener#display(com.jogamp.opengl.GLAutoDrawable, int) upstream.display(..)}.</li>
* </ul></li>
@@ -89,7 +89,7 @@ public interface StereoDeviceRenderer {
/**
* Distortion Bit: Timewarp distortion technique to predict
- * {@link EyePose} movement to reduce latency.
+ * {@link ViewerPose} movement to reduce latency.
* <p>
* FIXME: Explanation needs refinement!
@@ -219,7 +219,7 @@ public interface StereoDeviceRenderer {
* Begin stereoscopic post-processing, see {@link #ppAvailable()}.
* <p>
- * {@link #updateEyePose(int)} for both eyes must be called upfront
+ * {@link #updateViewerPose(int)} for both eyes must be called upfront
* when rendering upstream {@link StereoGLEventListener}.
* </p>
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoGLEventListener.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoGLEventListener.java
index c1a06796c..59f38f2af 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoGLEventListener.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoGLEventListener.java
@@ -51,7 +51,7 @@ public interface StereoGLEventListener extends CustomGLEventListener {
* </p>
* <p>
* The client shall also update it's projection- and modelview matrices according
* to the given {@link EyeParameter} and {@link EyePose}.
* </p>
* <p>
* For efficiency the GL viewport has already been updated
@@ -64,11 +64,11 @@ public interface StereoGLEventListener extends CustomGLEventListener {
* @param width viewport width in pixel units
* @param height viewport height in pixel units
* @param eyeParam constant eye parameter, i.e. FOV and IPD
* @param eyePose current eye position and orientation
* @see FloatUtil#makePerspective(float[], int, boolean, com.jogamp.opengl.math.FloatUtil.FovHVHalves, float, float)
*/
public void reshapeForEye(final GLAutoDrawable drawable, final int x, final int y, final int width, final int height,
- final EyeParameter eyeParam, final EyePose eyePose);
+ final EyeParameter eyeParam, final ViewerPose viewerPose);
}
@@ -51,7 +51,7 @@ public interface StereoGLEventListener extends CustomGLEventListener {
* <p>
* The client shall also update it's projection- and modelview matrices according
- * to the given {@link EyeParameter} and {@link EyePose}.
+ * to the given {@link EyeParameter} and {@link ViewerPose}.
* </p>
* <p>
@@ -64,11 +64,11 @@ public interface StereoGLEventListener extends CustomGLEventListener {
* @param height viewport height in pixel units
* @param eyeParam constant eye parameter, i.e. FOV and IPD
- * @param eyePose current eye position and orientation
+ * @param viewerPose current viewer position and orientation
* @see FloatUtil#makePerspective(float[], int, boolean, com.jogamp.opengl.math.FloatUtil.FovHVHalves, float, float)
*/
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoUtil.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoUtil.java
index b0ca4ddb2..b6f76a343 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoUtil.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/StereoUtil.java
@@ -97,23 +97,51 @@ public class StereoUtil {
return sb.toString();
}
+ /** See {@link StereoDevice#getSupportedSensorBits()} and {@link StereoDevice#getEnabledSensorBits()}. */
+ public static boolean usesOrientationSensor(final int sensorBits) { return 0 != ( sensorBits & StereoDevice.SENSOR_ORIENTATION ) ; }
+ /** See {@link StereoDevice#getSupportedSensorBits()} and {@link StereoDevice#getEnabledSensorBits()}. */
+ public static boolean usesYawCorrectionSensor(final int sensorBits) { return 0 != ( sensorBits & StereoDevice.SENSOR_YAW_CORRECTION ) ; }
+ /** See {@link StereoDevice#getSupportedSensorBits()} and {@link StereoDevice#getEnabledSensorBits()}. */
+ public static boolean usesPositionSensor(final int sensorBits) { return 0 != ( sensorBits & StereoDevice.SENSOR_POSITION ) ; }
+
+ /** See {@link StereoDevice#getSupportedSensorBits()} and {@link StereoDevice#getEnabledSensorBits()}. */
+ public static String sensorBitsToString(final int sensorBits) {
+ boolean appendComma = false;
+ final StringBuilder sb = new StringBuilder();
+ if( usesOrientationSensor(sensorBits) ) {
+ if( appendComma ) { sb.append(", "); };
+ sb.append("orientation"); appendComma=true;
+ }
+ if( usesYawCorrectionSensor(sensorBits) ) {
+ if( appendComma ) { sb.append(", "); };
+ sb.append("yaw-corr"); appendComma=true;
+ }
+ if( usesPositionSensor(sensorBits) ) {
+ if( appendComma ) { sb.append(", "); };
+ sb.append("position"); appendComma=true;
+ }
+ return sb.toString();
+ }
+
/**
* Calculates the <i>Side By Side</i>, SBS, projection- and modelview matrix for one eye.
* <p>
* {@link #updateEyePose(int)} must be called upfront.
* </p>
* <p>
* This method merely exist as an example implementation to compute the matrices,
* which shall be adopted by the
* {@link CustomGLEventListener#reshape(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, EyePose) upstream client code}.
* </p>
- * @param eyeNum eye denominator
+ * @param viewerPose
+ * @param eye
* @param zNear frustum near value
* @param zFar frustum far value
* @param mat4Projection float[16] projection matrix result
* @param mat4Modelview float[16] modelview matrix result
*/
- public static void getSBSUpstreamPMV(final Eye eye, final float zNear, final float zFar,
+ public static void getSBSUpstreamPMV(final ViewerPose viewerPose, final Eye eye,
+ final float zNear, final float zFar,
final float[] mat4Projection, final float[] mat4Modelview) {
final float[] mat4Tmp1 = new float[16];
final float[] mat4Tmp2 = new float[16];
@@ -122,7 +150,6 @@ public class StereoUtil {
final float[] vec3Tmp3 = new float[3];
final EyeParameter eyeParam = eye.getEyeParameter();
- final EyePose eyePose = eye.getLastEyePose();
//
// Projection
@@ -135,10 +162,10 @@ public class StereoUtil {
final Quaternion rollPitchYaw = new Quaternion();
// private final float eyeYaw = FloatUtil.PI; // 180 degrees in radians
// rollPitchYaw.rotateByAngleY(eyeYaw);
- final float[] shiftedEyePos = rollPitchYaw.rotateVector(vec3Tmp1, 0, eyePose.position, 0);
+ final float[] shiftedEyePos = rollPitchYaw.rotateVector(vec3Tmp1, 0, viewerPose.position, 0);
VectorUtil.addVec3(shiftedEyePos, shiftedEyePos, eyeParam.positionOffset);
- rollPitchYaw.mult(eyePose.orientation);
+ rollPitchYaw.mult(viewerPose.orientation);
final float[] up = rollPitchYaw.rotateVector(vec3Tmp2, 0, VectorUtil.VEC3_UNIT_Y, 0);
final float[] forward = rollPitchYaw.rotateVector(vec3Tmp3, 0, VectorUtil.VEC3_UNIT_Z_NEG, 0);
final float[] center = VectorUtil.addVec3(forward, shiftedEyePos, forward);
@@ -97,23 +97,51 @@ public class StereoUtil {
* Calculates the <i>Side By Side</i>, SBS, projection- and modelview matrix for one eye.
* <p>
- * {@link #updateEyePose(int)} must be called upfront.
+ * {@link #updateViewerPose(int)} must be called upfront.
* </p>
* <p>
@@ -97,23 +97,51 @@ public class StereoUtil {
* This method merely exist as an example implementation to compute the matrices,
* which shall be adopted by the
- * {@link CustomGLEventListener#reshape(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, EyePose) upstream client code}.
+ * {@link CustomGLEventListener#reshape(com.jogamp.opengl.GLAutoDrawable, int, int, int, int, EyeParameter, ViewerPose) upstream client code}.
* </p>
diff --git a/src/jogl/classes/com/jogamp/opengl/util/stereo/EyePose.java b/src/jogl/classes/com/jogamp/opengl/util/stereo/ViewerPose.java
index aa64ff130..10ee4c994 100644
--- a/src/jogl/classes/com/jogamp/opengl/util/stereo/EyePose.java
+++ b/src/jogl/classes/com/jogamp/opengl/util/stereo/ViewerPose.java
@@ -30,29 +30,34 @@ package com.jogamp.opengl.util.stereo;
import com.jogamp.opengl.math.Quaternion;
/**
* Position and orientation of one eye.
*/
-public final class EyePose {
- /** Eye number, <code>0</code> for the left eye and <code>1</code> for the right eye. */
- public final int number;
-
- /** float[3] eye position vector. */
+public final class ViewerPose {
+ /**
+ * float[3] position of viewer in meter.
+ * <p>
+ * Apply the following to resolve the actual eye position:
+ * <ul>
+ * <li>{@link EyeParameter#positionOffset positionOffset} for head.</li>
+ * <li>[{@link EyeParameter#distNoseToPupilX distNoseToPupilX}, {@link EyeParameter#distMiddleToPupilY distMiddleToPupilY}, {@link EyeParameter#eyeReliefZ eyeReliefZ}] for pupil</li>
+ * </ul>
+ * </p>
+ */
public final float[] position;
- /** Eye orientation */
+ /** Orientation of viewer. */
public final Quaternion orientation;
- public EyePose(final int number) {
- this.number = number;
+ public ViewerPose() {
this.position = new float[3];
this.orientation = new Quaternion();
}
- public EyePose(final int number, final float[] position, final Quaternion orientation) {
- this(number);
+ public ViewerPose(final float[] position, final Quaternion orientation) {
+ this();
set(position, orientation);
}
- /** Set position and orientation of this instance. */
+ /** Set {@link #position} and {@link #orientation}. */
public final void set(final float[] position, final Quaternion orientation) {
System.arraycopy(position, 0, this.position, 0, 3);
this.orientation.set(orientation);
@@ -64,6 +69,6 @@ public final class EyePose {
position[2] = posZ;
}
public final String toString() {
- return "EyePose[num "+number+", pos["+position[0]+", "+position[1]+", "+position[2]+"], "+orientation+"]";
+ return "ViewerPose[pos["+position[0]+", "+position[1]+", "+position[2]+"], "+orientation+"]";
}
} \ No newline at end of file
@@ -30,29 +30,34 @@ package com.jogamp.opengl.util.stereo;
import com.jogamp.opengl.math.Quaternion;
/**
- * Position and orientation of one eye.
+ * {@link #position} and {@link #orientation} of viewer.
*/